@@ -27,6 +27,7 @@ import copy
 import collections
 
 
+
 class ObjectiveParam(BaseParam):
     """
     Define objective parameters that used in federated ml.
@@ -98,7 +99,12 @@ class DecisionTreeParam(BaseParam):
     ----------
     criterion_method : str, accepted "xgboost" only, the criterion function to use, default: 'xgboost'
 
-    criterion_params: list, should be non empty and first element is float-number, default: 0.1.
+    criterion_params: list or dict, should be non empty and elements are float-numbers,
+                      if a list is offered, the first one is l2 regularization value, and the second one is
+                      l1 regularization value.
+                      if a dict is offered, make sure it contains key 'l1', and 'l2'.
+                      l1, l2 regularization values are non-negative floats.
+                      default: [0.1, 0] or {'l1':0, 'l2':0,1}
 
     max_depth: int, positive integer, the max depth of a decision tree, default: 3
 
@@ -106,6 +112,8 @@ class DecisionTreeParam(BaseParam):
 
     min_impurity_split: float, least gain of a single split need to reach, default: 1e-3
 
+    min_child_weight: float, sum of hessian needed in child nodes. default is 0
+
     min_leaf_node: int, when samples no more than min_leaf_node, it becomes a leave, default: 1
 
     max_split_nodes: int, positive integer, we will use no more than max_split_nodes to
@@ -121,13 +129,18 @@ class DecisionTreeParam(BaseParam):
     zero_as_missing: bool, accepted True, False only, regard 0 as missing value or not,
                      will be use only if use_missing=True, default: False
 
+    deterministic: bool, ensure stability when computing histogram. Set this to true to ensure stable result when using
+                         same data and same parameter. But it may slow down computation.
+
     """
 
-    def __init__(self, criterion_method="xgboost", criterion_params=[0.1], max_depth=3,
+    def __init__(self, criterion_method="xgboost", criterion_params=[0.1, 0], max_depth=3,
                  min_sample_split=2, min_impurity_split=1e-3, min_leaf_node=1,
                  max_split_nodes=consts.MAX_SPLIT_NODES, feature_importance_type="split",
-                 n_iter_no_change=True, tol=0.001,
-                 use_missing=False, zero_as_missing=False,):
+                 n_iter_no_change=True, tol=0.001, min_child_weight=0,
+                 use_missing=False, zero_as_missing=False, deterministic=False):
+
+        super(DecisionTreeParam, self).__init__()
 
         self.criterion_method = criterion_method
         self.criterion_params = criterion_params
@@ -135,12 +148,14 @@ class DecisionTreeParam(BaseParam):
         self.min_sample_split = min_sample_split
         self.min_impurity_split = min_impurity_split
         self.min_leaf_node = min_leaf_node
+        self.min_child_weight = min_child_weight
         self.max_split_nodes = max_split_nodes
         self.feature_importance_type = feature_importance_type
         self.n_iter_no_change = n_iter_no_change
         self.tol = tol
         self.use_missing = use_missing
         self.zero_as_missing = zero_as_missing
+        self.deterministic = deterministic
 
     def check(self):
         descr = "decision tree param"
@@ -149,15 +164,21 @@ class DecisionTreeParam(BaseParam):
                                                              ["xgboost"],
                                                              descr)
 
-        if type(self.criterion_params).__name__ != "list":
-            raise ValueError("decision tree param's criterion_params {} not supported, should be list".format(
-                self.criterion_params))
-
         if len(self.criterion_params) == 0:
             raise ValueError("decisition tree param's criterio_params should be non empty")
 
-        if type(self.criterion_params[0]).__name__ not in ["int", "long", "float"]:
-            raise ValueError("decision tree param's criterion_params element shoubld be numeric")
+        if type(self.criterion_params) == list:
+            assert len(self.criterion_params) == 2, 'length of criterion_param should be 2: l1, l2 regularization ' \
+                                                    'values are needed'
+            self.check_nonnegative_number(self.criterion_params[0], 'l2 reg value')
+            self.check_nonnegative_number(self.criterion_params[1], 'l1 reg value')
+
+        elif type(self.criterion_params) == dict:
+            assert 'l1' in self.criterion_params and 'l2' in self.criterion_params, 'l1 and l2 keys are needed in ' \
+                                                                                    'criterion_params dict'
+            self.criterion_params = [self.criterion_params['l2'], self.criterion_params['l1']]
+        else:
+            raise ValueError('criterion_params should be a dict or a list contains l1, l2 reg value')
 
         if type(self.max_depth).__name__ not in ["int", "long"]:
             raise ValueError("decision tree param's max_depth {} not supported, should be integer".format(
@@ -194,6 +215,9 @@ class DecisionTreeParam(BaseParam):
                                                                     ["split", "gain"],
                                                                     descr)
 
+        self.check_nonnegative_number(self.min_child_weight, 'min_child_weight')
+        self.check_boolean(self.deterministic, 'deterministic')
+
         return True
 
 
@@ -211,7 +235,7 @@ class BoostingParam(BaseParam):
 
         num_trees : int, accepted int, float only, the max number of boosting round. default: 5
 
-        subsample_feature_rate : float, a float-number in [0, 1], default: 0.8
+        subsample_feature_rate : float, a float-number in [0, 1], default: 1.0
 
         n_iter_no_change : bool,
             when True and residual error less than tol, tree building process will stop. default: True
@@ -231,7 +255,7 @@ class BoostingParam(BaseParam):
                  learning_rate=0.3, num_trees=5, subsample_feature_rate=1, n_iter_no_change=True,
                  tol=0.0001, bin_num=32,
                  predict_param=PredictParam(), cv_param=CrossValidationParam(),
-                 validation_freqs=None, metrics=None, subsample_random_seed=None,
+                 validation_freqs=None, metrics=None, random_seed=100,
                  binning_error=consts.DEFAULT_RELATIVE_ERROR):
 
         super(BoostingParam, self).__init__()
@@ -248,7 +272,7 @@ class BoostingParam(BaseParam):
         self.cv_param = copy.deepcopy(cv_param)
         self.validation_freqs = validation_freqs
         self.metrics = metrics
-        self.subsample_random_seed = subsample_random_seed
+        self.random_seed = random_seed
         self.binning_error = binning_error
 
     def check(self):
@@ -292,8 +316,8 @@ class BoostingParam(BaseParam):
         if self.metrics is not None and not isinstance(self.metrics, list):
             raise ValueError("metrics should be a list")
 
-        if self.subsample_random_seed is not None:
-            assert type(self.subsample_random_seed) == int and self.subsample_random_seed >= 0, 'random seed must be an integer >= 0'
+        if self.random_seed is not None:
+            assert type(self.random_seed) == int and self.random_seed >= 0, 'random seed must be an integer >= 0'
 
         self.check_decimal_float(self.binning_error, descr)
 
@@ -317,12 +341,12 @@ class HeteroBoostingParam(BoostingParam):
                  encrypted_mode_calculator_param=EncryptedModeCalculatorParam(),
                  predict_param=PredictParam(), cv_param=CrossValidationParam(),
                  validation_freqs=None, early_stopping_rounds=None, metrics=None, use_first_metric_only=False,
-                 subsample_random_seed=None, binning_error=consts.DEFAULT_RELATIVE_ERROR):
+                 random_seed=100, binning_error=consts.DEFAULT_RELATIVE_ERROR):
 
         super(HeteroBoostingParam, self).__init__(task_type, objective_param, learning_rate, num_trees,
                                                   subsample_feature_rate, n_iter_no_change, tol, bin_num,
                                                   predict_param, cv_param, validation_freqs, metrics=metrics,
-                                                  subsample_random_seed=subsample_random_seed,
+                                                  random_seed=random_seed,
                                                   binning_error=binning_error)
 
         self.encrypt_param = copy.deepcopy(encrypt_param)
@@ -366,9 +390,9 @@ class HeteroSecureBoostParam(HeteroBoostingParam):
 
         num_trees : int, accepted int, float only, the max number of trees to build. default: 5
 
-        subsample_feature_rate : float, a float-number in [0, 1], default: 0.8
+        subsample_feature_rate : float, a float-number in [0, 1], default: 1.0
 
-        subsample_random_seed: seed that controls feature subsample
+        random_seed: seed that controls all random functions
 
         n_iter_no_change : bool,
             when True and residual error less than tol, tree building process will stop. default: True
@@ -412,8 +436,22 @@ class HeteroSecureBoostParam(HeteroBoostingParam):
         complete_secure: bool, if use complete_secure, when use complete secure, build first tree using only guest
                         features
 
-        sparse_optimization: This parameter is abandoned due to the removal of IterativeAffine
+        sparse_optimization: bool, Available when encrypted method is 'iterativeAffine'
+                            An optimized mode for high-dimension, sparse data.
 
+        run_goss: bool, activate Gradient-based One-Side Sampling, which selects large gradient and small
+                   gradient samples using top_rate and other_rate.
+
+        top_rate: float, the retain ratio of large gradient data, used when run_goss is True
+
+        other_rate: float, the retain ratio of small gradient data, used when run_goss is True
+
+        cipher_compress_error： int between [0-15], default is None. The parameter to control pallier cipher compressing.
+                        When cipher compressing is enabled, communication cost will be reduced, algorithms may run f
+                        aster due to lower decrypt cost. However, performance will be influenced by the precision
+                        loss caused by cipher compress.
+                        'None' means disable cipher compress.
+                        A specified integer indicates the rounding decimal precision.
         """
 
     def __init__(self, tree_param: DecisionTreeParam = DecisionTreeParam(), task_type=consts.CLASSIFICATION,
@@ -424,16 +462,17 @@ class HeteroSecureBoostParam(HeteroBoostingParam):
                  encrypted_mode_calculator_param=EncryptedModeCalculatorParam(),
                  predict_param=PredictParam(), cv_param=CrossValidationParam(),
                  validation_freqs=None, early_stopping_rounds=None, use_missing=False, zero_as_missing=False,
-                 complete_secure=False, metrics=None, use_first_metric_only=False, subsample_random_seed=None,
+                 complete_secure=False, metrics=None, use_first_metric_only=False, random_seed=100,
                  binning_error=consts.DEFAULT_RELATIVE_ERROR,
-                 sparse_optimization=False):
+                 sparse_optimization=False, run_goss=False, top_rate=0.2, other_rate=0.1,
+                 cipher_compress_error=None, new_ver=True):
 
         super(HeteroSecureBoostParam, self).__init__(task_type, objective_param, learning_rate, num_trees,
                                                      subsample_feature_rate, n_iter_no_change, tol, encrypt_param,
                                                      bin_num, encrypted_mode_calculator_param, predict_param, cv_param,
                                                      validation_freqs, early_stopping_rounds, metrics=metrics,
                                                      use_first_metric_only=use_first_metric_only,
-                                                     subsample_random_seed=subsample_random_seed,
+                                                     random_seed=random_seed,
                                                      binning_error=binning_error)
 
         self.tree_param = tree_param
@@ -441,6 +480,11 @@ class HeteroSecureBoostParam(HeteroBoostingParam):
         self.use_missing = use_missing
         self.complete_secure = complete_secure
         self.sparse_optimization = sparse_optimization
+        self.run_goss = run_goss
+        self.top_rate = top_rate
+        self.other_rate = other_rate
+        self.cipher_compress_error = cipher_compress_error
+        self.new_ver = new_ver
 
     def check(self):
 
@@ -452,6 +496,34 @@ class HeteroSecureBoostParam(HeteroBoostingParam):
             raise ValueError('zero as missing should be bool type')
         self.check_boolean(self.complete_secure, 'complete_secure')
         self.check_boolean(self.sparse_optimization, 'sparse optimization')
+        self.check_boolean(self.run_goss, 'run goss')
+        self.check_decimal_float(self.top_rate, 'top rate')
+        self.check_decimal_float(self.other_rate, 'other rate')
+        self.check_boolean(self.new_ver, 'code version switcher')
+
+        if self.top_rate + self.other_rate >= 1:
+            raise ValueError('sum of top rate and other rate should be smaller than 1')
+
+        if self.cipher_compress_error is not None:
+            self.check_positive_integer(self.cipher_compress_error, 'cipher_compress_error')
+            if self.cipher_compress_error > 15:
+                raise ValueError('cipher compress error exceeds max value 15.')
+
+            # safety check
+            if self.encrypt_param.method != consts.PAILLIER:
+                LOGGER.warning('cipher compressing only supports Paillier, however, encrypt method is {}, '
+                               'this function will be disabled automatically'.
+                               format(self.encrypt_param.method))
+                self.cipher_compress_error = None
+
+            if self.task_type != consts.CLASSIFICATION:
+                LOGGER.warning('cipher compressing only supports classification tasks'
+                               'this function will be disabled automatically')
+                self.cipher_compress_error = None
+
+            if not self.new_ver:
+                LOGGER.warning('old version code does not support cipher compressing')
+                self.cipher_compress_error = None
 
         return True
 
@@ -467,7 +539,8 @@ class HeteroFastSecureBoostParam(HeteroSecureBoostParam):
                  predict_param=PredictParam(), cv_param=CrossValidationParam(),
                  validation_freqs=None, early_stopping_rounds=None, use_missing=False, zero_as_missing=False,
                  complete_secure=False, tree_num_per_party=1, guest_depth=1, host_depth=1, work_mode='mix', metrics=None,
-                 sparse_optimization=False, subsample_random_seed=None, binning_error=consts.DEFAULT_RELATIVE_ERROR):
+                 sparse_optimization=False, random_seed=100, binning_error=consts.DEFAULT_RELATIVE_ERROR,
+                 cipher_compress_error=None, new_ver=True, run_goss=False, top_rate=0.2, other_rate=0.1):
 
         """
         work_mode：
@@ -490,9 +563,12 @@ class HeteroFastSecureBoostParam(HeteroSecureBoostParam):
                                                          encrypt_param, bin_num, encrypted_mode_calculator_param,
                                                          predict_param, cv_param, validation_freqs, early_stopping_rounds,
                                                          use_missing, zero_as_missing, complete_secure, metrics=metrics,
-                                                         subsample_random_seed=subsample_random_seed,
+                                                         random_seed=random_seed,
                                                          sparse_optimization=sparse_optimization,
-                                                         binning_error=binning_error)
+                                                         binning_error=binning_error,
+                                                         cipher_compress_error=cipher_compress_error,
+                                                         new_ver=new_ver,
+                                                         run_goss=run_goss, top_rate=top_rate, other_rate=other_rate)
 
         self.tree_num_per_party = tree_num_per_party
         self.guest_depth = guest_depth
@@ -523,7 +599,7 @@ class HomoSecureBoostParam(BoostingParam):
                  objective_param=ObjectiveParam(),
                  learning_rate=0.3, num_trees=5, subsample_feature_rate=1, n_iter_no_change=True,
                  tol=0.0001, bin_num=32, predict_param=PredictParam(), cv_param=CrossValidationParam(),
-                 validation_freqs=None, use_missing=False, zero_as_missing=False, subsample_random_seed=None,
+                 validation_freqs=None, use_missing=False, zero_as_missing=False, random_seed=100,
                  binning_error=consts.DEFAULT_RELATIVE_ERROR
                  ):
         super(HomoSecureBoostParam, self).__init__(task_type=task_type,
@@ -537,7 +613,7 @@ class HomoSecureBoostParam(BoostingParam):
                                                    predict_param=predict_param,
                                                    cv_param=cv_param,
                                                    validation_freqs=validation_freqs,
-                                                   subsample_random_seed=subsample_random_seed,
+                                                   random_seed=random_seed,
                                                    binning_error=binning_error
                                                    )
         self.use_missing = use_missing
@@ -553,4 +629,3 @@ class HomoSecureBoostParam(BoostingParam):
             raise ValueError('zero as missing should be bool type')
         return True
 
-
