@@ -118,36 +118,38 @@ class JobInvoker(object):
         party_id = str(party_id)
         start_time = time.time()
         pre_cpn = None
-        LOGGER.info(f"Job id is {job_id}")
+        LOGGER.info(f"Job id is {job_id}\n")
         while True:
             ret_code, ret_msg, data = self.query_job(job_id, role, party_id)
             status = data["f_status"]
             if status == JobStatus.SUCCESS:
                 # print("job is success!!!")
                 elapse_seconds = timedelta(seconds=int(time.time() - start_time))
-                sys.stdout.write(f"\n\r")
+                # sys.stdout.write(f"\n\r")
                 LOGGER.info(f"Job is success!!! Job id is {job_id}")
                 LOGGER.info(f"Total time: {elapse_seconds}")
                 return StatusCode.SUCCESS
 
             elif status == JobStatus.FAILED:
-                sys.stdout.write(f"\n\r")
+                # sys.stdout.write(f"\n\r")
+                # LOGGER.info(f"\n\r")
                 raise ValueError(f"Job is failed, please check out job {job_id} by fate board or fate_flow cli")
 
             elif status == JobStatus.WAITING:
                 elapse_seconds = timedelta(seconds=int(time.time() - start_time))
-                sys.stdout.write(f"Job is still waiting, time elapse: {elapse_seconds}\r")
-                sys.stdout.flush()
+                # sys.stdout.write(f"\r")
+                # sys.stdout.flush()
+                LOGGER.info(f"\x1b[80D\x1b[1A\x1b[KJob is still waiting, time elapse: {elapse_seconds}")
 
             elif status == JobStatus.CANCELED:
                 elapse_seconds = timedelta(seconds=int(time.time() - start_time))
-                sys.stdout.write(f"\n\r")
+                # sys.stdout.write(f"\n\r")
                 LOGGER.info(f"Job is canceled, time elapse: {elapse_seconds}\r")
                 return StatusCode.CANCELED
 
             elif status == JobStatus.TIMEOUT:
                 elapse_seconds = timedelta(seconds=int(time.time() - start_time))
-                sys.stdout.write(f"\n\r")
+                # sys.stdout.write(f"\n\r")
                 raise ValueError(f"Job is timeout, time elapse: {elapse_seconds}\r")
 
             elif status == JobStatus.RUNNING:
@@ -166,10 +168,12 @@ class JobInvoker(object):
                         cpn.append(cpn_data["f_component_name"])
 
                 if cpn != pre_cpn:
-                    sys.stdout.write(f"\n \r")
+                    LOGGER.info(f"\r")
                     pre_cpn = cpn
-                sys.stdout.write(f"Running component {cpn}, time elapse: {elapse_seconds}\r")
-                sys.stdout.flush()
+                # sys.stdout.write(f"\r")
+                # sys.stdout.flush()
+                LOGGER.info(f"\x1b[80D\x1b[1A\x1b[KRunning component {cpn}, time elapse: {elapse_seconds}")
+
             else:
                 raise ValueError(f"Unknown status: {status}")
 
@@ -432,21 +436,3 @@ class JobInvoker(object):
             raise ValueError("Cannot get predict dsl, error msg is {}".format(result["retmsg"]))
         else:
             return result["data"]
-
-    """
-    def get_predict_dsl(self, train_dsl, cpn_list, version):
-        result = None
-        with tempfile.TemporaryDirectory() as job_dir:
-            train_dsl_path = os.path.join(job_dir, "train_dsl.json")
-            with open(train_dsl_path, "w") as fout:
-                fout.write(json.dumps(train_dsl))
-
-            result = self.client.job.generate_dsl(train_dsl_path=train_dsl_path, cpn_list=cpn_list, version=version)
-
-        if result is None or 'retcode' not in result:
-            raise ValueError("Call flow generate dsl is failed, check if fate_flow server is start!")
-        elif result["retcode"] != 0:
-            raise ValueError("Cannot generate predict dsl, error msg is {}".format(result["retmsg"]))
-        else:
-            return result["data"]
-    """
\ No newline at end of file
